#!/bin/sh
# Post installation for debian 3.1 installation

# THis is the module name. Should be valid.
HPTMOD=rr3740a

#
# Do NOT edit the following in most cases
#
ININITRD=0

if test "${HPTMOD}set" = "set" ; then
	echo "Error! Which module to install?";
	exit 1;
fi

if [ ! -d /lib/modules/ ]; then
	echo "Error! Can not find any system module installation directory!";
	exit 1;
fi

RELEASES=$( ls /lib/modules/ )

if test "${RELEASES}"set = "set" ; then
	echo "Error! No kernel package has been installed!";
	exit 1;
fi

if [ "`/bin/uname -m`" = "x86_64" ];then
    ARCH=x86_64
else
    ARCH=i386
fi

VERSION=$( ls /lib/modules/ | cut -c1-3 | uniq )
if which mkinitramfs >/dev/null 2>/dev/null ; then
	mkinitrd=mkinitramfs
else
	mkinitrd=mkinitrd
fi


if ( grep "^$HPTMOD" -s -q /etc/${mkinitrd}/modules ); then # debian 3.1, ubuntu 5.10 (6.x? 7.x?)
	ININITRD=1
elif ( grep "^MODULES=most" -s -q /etc/initramfs-tools/initramfs.conf ); then # debian 4.0
	ININITRD=1
	echo "ROOTDELAY=180" >> /etc/initramfs-tools/initramfs.conf
elif ( grep "^$HPTMOD" -s -q /etc/initramfs-tools/modules ) ; then # debian 4.0
	ININITRD=1
else
	echo "No initrd config file found, will not rebuild initrd image."
fi

for RELEASE in ${RELEASES}; do
	if [ -f boot/${HPTMOD}${RELEASE}${ARCH}.ko.gz -a -d /lib/modules/${RELEASE}/kernel ] ; then
		if test $HPTMOD = "hptmv" || test $HPTMOD = "hptmv6" || test $HPTMOD = "rr174x" || test $HPTMOD = "rr2310_00"; then
			find /lib/modules/${RELEASE} -name 'sata_mv.*' -delete
		fi
		if test $HPTMOD = "rr272x_1x" || test $HPTMOD = "rr274x_3x" ||  test $HPTMOD = "rr276x" || test $HPTMOD = "rr278x" ; then
			find /lib/modules/${RELEASE} -name 'mvsas.*' -delete
		fi
		if test $HPTMOD = "hptiop"; then
			find /lib/modules/${RELEASE} -name 'hptiop.*' -delete
		fi
		if test $HPTMOD = "hpt374" || test $HPTMOD = "hpt37x2"; then
			find /lib/modules/${RELEASE} -name 'pata_hpt3*.ko' -delete
			find /lib/modules/${RELEASE} -name 'hpt366.ko' -delete
		fi
		gzip -dc boot/$HPTMOD${RELEASE}${ARCH}.ko.gz > /lib/modules/${RELEASE}/kernel/drivers/scsi/${HPTMOD}.ko	
		depmod -a ${RELEASE}
		if test $ININITRD = "1" ; then
			echo "Update initrd file /boot/initrd.img-${RELEASE} for ${RELEASE}"
			mv /boot/initrd.img-${RELEASE} /boot/initrd.img-${RELEASE}.bak
			${mkinitrd} -o /boot/initrd.img-${RELEASE} ${RELEASE}
		fi
	fi
done


if test $ININITRD = "1" ; then
	# in case driver not in initrd and udev not load the driver module
	if [ ! -f /etc/modules ] || ( ! grep "^$HPTMOD" /etc/modules -s -q ); then
		echo $HPTMOD >> /etc/modules
		echo sd_mod >> /etc/modules
	fi
	echo "Please reboot the system to use the new driver module."
	exit 0
elif [ ! -f /etc/modules ] || ( ! grep "^$HPTMOD" /etc/modules -s -q ); then
	echo $HPTMOD >> /etc/modules
	echo sd_mod >> /etc/modules
	if test $HPTMOD = "hptmv" || test $HPTMOD = "hptmv6" || test $HPTMOD = "rr174x" || test $HPTMOD = "rr2310_00" ; then
		if ( lsmod | grep sata_mv -s -q ); then
			echo "Unloading conflicting module sata_mv..."
			rmmod sata_mv
			if ( lsmod | grep sata_mv -s -q ) ; then
				echo "Failed to unload module sata_mv, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	elif test $HPTMOD = "hpt374" || test $HPTMOD = "hpt37x2"; then
		if ( lsmod | grep hpt366 -s -q ); then
			echo "Unloading conflicting module hpt366..."
			rmmod hpt366
			if ( lsmod | grep hpt366 -s -q ) ; then
				echo "Failed to unload module hpt366, it may be in use, please reboot the system."
				exit 0
			fi
		fi
		if ( lsmod | grep pata_hpt37x -s -q ); then
			echo "Unloading conflicting module pata_hpt37x..."
			rmmod pata_hpt37x
			if ( lsmod | grep pata_hpt37x -s -q ) ; then
				echo "Failed to unload module pata_hpt37x, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	elif test $HPTMOD = "hptiop" ; then
		if ( lsmod | grep hptiop -s -q ); then
			echo "Unloading previous loading module hptiop..."
			rmmod hptiop
			if ( lsmod | grep hptiop -s -q ) ; then
				echo "Failed to unload module hptiop, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	elif test $HPTMOD = "rr272x_1x" || test $HPTMOD = "rr274x_3x" || test $HPTMOD = "rr276x" || test $HPTMOD = "rr278x" ; then
		if ( lsmod | grep mvsas -s -q ); then
			echo "Unloading previous loading module mvsas..."
			rmmod mvsas
			if ( lsmod | grep mvsas -s -q ) ; then
				echo "Failed to unload module mvsas, it may be in use, please reboot the system."
				exit 0
			fi
		fi
	fi
	echo "Loading $product driver module $HPTMOD."
	modprobe $HPTMOD
	modprobe sd_mod
	exit 0
fi
